<?php
/**
 * Atom Helper
 *
 * PHP Version 5.3.1とCakePHP 1.3.11で動作を確認。
 *
 * @author http://purl.org/meta/me/
 * @version Atom Helper 1.0 2011-08-17
 * @license Creative Commons License CC BY (http://creativecommons.org/licenses/by/3.0/)
 */

App::import('Helper', 'Xml');

/**
 * AtomHelperクラスはAtomの作成を手助けするヘルパーです。
 *
 * @link http://www.ietf.org/rfc/rfc4287.txt
 */
class AtomHelper extends XmlHelper {

	/**
	 * entry要素以下を一時的に溜めておくメンバー変数
	 *
	 * @var array
	 * @access protected
	 * @see getEntry(), setEntry()
	 */
	var $_entry = array();

	/**
	 * feed要素以下を一時的に溜めておくメンバー変数
	 *
	 * @var array
	 * @access protected
 	 * @see getFeed(), setFeed()
	 */
	var $_feed = array();

	/**
	 * author要素を生成します。(ja)
	 *
	 * Generate author element.(en)
	 *
	 * <code>
	 * $atom->author(array(
	 *     'name' => 'Mark Pilgrim',
	 *     'uri' => 'http://example.org/',
	 *     'email' => 'f8dy@example.com'
	 * ));
	 *
	 * $atom->author('
	 *     <name>Mark Pilgrim</name>
	 *     <uri>http://example.org/</uri>
	 *     <email>f8dy@example.com</email>
	 * ');
	 *
	 * # 出力すると以下のようになります。
	 * # <author>
	 * #     <name>Mark Pilgrim</name>
	 * #     <uri>http://example.org/</uri>
	 * #     <email>f8dy@example.com</email>
	 * # </author>
	 * </code>
	 *
	 * @param mixed $content author要素の内容。配列または文字列(ja) author element content. array or string(en)
	 * @param array $attributes author要素の属性(ja) The attributes of the author element(en)
	 * @return string author要素(ja) author element(en)
	 * @access public
	 */
	function author($content = array(), $attributes = array()) {
		$out = $this->personConstruct('author', $content, $attributes);

		return $out;
	}

	/**
	 * category要素を生成します。(ja)
	 *
	 * Generate category element.(en)
	 *
	 * @param array $attributes category要素の属性(ja) The attributes of the category element(en)
	 * @return string category要素(ja) category element(en)
	 * @access public
	 */
	function category($attributes = array()) {
		$attributes += array('#urn' => false);

		if(isset($attributes['scheme']) && !$attributes['#urn']) {
			$attributes['scheme'] = $this->url($attributes['scheme'], true);
		}

		unset($attributes['#urn']);

		return $this->tag('generator', null, $attributes);
	}

	/**
	 * content要素を生成します。(ja)
	 *
	 * Generate content element.(en)
	 *
	 * <code>
	 * $atom->content(
	 *     '<p><i>[Update: The Atom draft is finished.]</i></p>',
	 *     array('type' => 'xhtml', 'xml:lang' => 'en', 'xml:base' => 'http://diveintomark.org/')
	 * );
	 *
	 * # 出力すると以下のようになります。
	 * # <content type="xhtml" xml:lang="en" xml:base="http://diveintomark.org/">
	 * #    <div xmlns="http://www.w3.org/1999/xhtml">
	 * #        <p><i>[Update: The Atom draft is finished.]</i></p>
	 * #    </div>
	 * # </content>
	 * </code>
	 *
	 * @param string $content content要素の内容(ja) content element content(en)
	 * @param array $attributes content要素の属性(ja) The attributes of the content element(en)
	 * @return string content要素(ja) content element(en)
	 * @access public
	 */
	function content($content = null, $attributes = array()) {
		$attributes += array('#escape' => true);

		if(isset($attributes['src'])) {
			$attributes['src'] = $this->url($attributes['src'], true);
		} else {
			if(!isset($attributes['type'])) {
				$attributes['type'] = 'text';
			}

			if($attributes['type'] === 'xhtml') {
				// type属性が"xhtml"の場合の処理

				$content = $this->tag('div', $content, array('xmlns' => 'http://www.w3.org/1999/xhtml'));

				$attributes['#escape'] = false;
			} elseif(preg_match('/^.+?(\+xml|\/xml)$/i', $attributes['type'])) {
				// type属性の末尾が"+xml"か"/xml"の場合の処理

				$attributes['#escape'] = false;
			} elseif($attributes['type'] !== 'text' && $attributes['type'] !== 'html' && stripos($attributes['type'], 'text/') === 0) {
				// type属性が規定のものではない場合の処理

				$content = base64_encode($content);
			}
		}

		return $this->tag('content', $content, $attributes);
	}

	/**
	 * contributor要素を生成します。(ja)
	 *
	 * Generate contributor element.(en)
	 *
	 * <code>
	 * $atom->contributor(array(
	 *     'name' => 'Sam Ruby'
	 * ));
	 *
	 * $atom->contributor('
	 *     <name>Sam Ruby</name>
	 * ');
	 *
	 * # 出力すると以下のようになります。
	 * # <contributor>
	 * #     <name>Sam Ruby</name>
	 * # </contributor>
	 * </code>
	 *
	 * @param mixed $content contributor要素の内容(ja) contributor element content(en) 配列または文字列
	 * @param array $attributes contributor要素の属性(ja) The attributes of the contributor element(en)
	 * @return string contributor要素(ja) contributor element(en)
	 * @access public
	 */
	function contributor($content = array(), $attributes = array()) {
		$out = $this->personConstruct('contributor', $content, $attributes);

		return $out;
	}

	/**
	 * 現在のURLを返します。(ja)
	 *
	 * Returns the current URL.(en)
	 *
	 * <code>
	 * $atom->currentUrl();
	 * </code>
	 *
	 * @return string 現在のURL(ja) current URL(en)
	 * @access public
	 */
	function currentUrl() {
		$xmlBase = env('HTTP_HOST') . env('REQUEST_URI');

		if(env('HTTPS')) {
			return 'https://' . $xmlBase;
		}

		return 'http://' . $xmlBase;
	}

	/**
	 * タイムスタンプをAtomの日付形式に変換します。
	 *
	 * <code>
	 * $atom->date(1071340202);
	 *
	 * # 出力すると以下のようになります。
	 * # 2003-12-13T18:30:02+00:00
	 * </code>
	 *
	 * @param integer $timestamp タイムスタンプ(ja) Timestamp(en)
	 * @return string date-time形式(ja) date-time format(en)
	 * @access public
	 */
	function date($timestamp = null) {
		if(!$timestamp) {
			$timestamp = time();
		}

		if(ctype_digit((string) $timestamp)) {
			return date(DATE_ATOM, $timestamp);
		}

		return $timestamp;
	}

	/**
	 * email要素を生成します。(ja)
	 *
	 * Generate email element.(en)
	 *
	 * <code>
	 * $atom->email('f8dy@example.com');
	 *
	 * # 出力すると以下のようになります。
	 * # <email>f8dy@example.com</email>
	 * </code>
	 *
	 * @param string $content email要素の内容(ja) email element content(en)
	 * @param array $attributes email要素の属性(ja) The attributes of the email element(en)
	 * @return string email要素(ja) email element(en)
	 * @access public
	 */
	function email($content, $attributes = array()) {
		$attributes += array('#escape' => true);
		return $this->tag('email', $content, $attributes);
	}

	/**
	 * 終了タグを生成します。(ja)
	 *
	 * Generate end tag.(en)
	 *
	 * <code>
	 * $atom->endTag('entry');
	 *
	 * $atom->endTag('feed');
	 *
	 * # 出力すると以下のようになります。
	 * # </entry>
	 * #
	 * # </feed>
	 * </code>
	 *
	 * @param string $name 要素の名前(ja) The name of the end tag(en)
	 * @return string 終了タグ(ja) End tag(en)
	 * @access public
	 */
	function endTag($name) {
		$out = '</' . $name . '>';

		return $out;
	}

	/**
	 * entry要素を生成します。(ja)
	 *
	 * Generate entry element.(en)
	 *
	 * <code>
	 * $atom->entry('<title>Example</title>...');
	 *
	 * // atom::setEntryで溜めたものを吐き出します。
	 * $atom->entry();
	 * </code>
	 *
	 * @param string $content entry要素の内容(ja) entry element content(en)
	 * @param array $attributes entry要素の属性(ja) The attributes of the entry element(en)
	 * @return string entry要素(ja) entry element(en)
	 * @access public
	 */
	function entry($content = null, $attributes = array()) {
		if(!$content) {
			$content = $this->getEntry();
		}

		return $this->tag('entry', $content, $attributes);
	}

	/**
	 * feed要素を生成します。(ja)
	 *
	 * Generate feed element.(en)
	 *
	 * <code>
	 * $atom->feed('<title>Example</title>...');
	 *
	 * // atom::setFeedで溜めたものを吐き出します。
	 * $atom->feed();
	 * </code>
	 *
	 * @param string $content feed要素の内容(ja) feed element content(en)
	 * @param array $attributes feed要素の属性(ja) The attributes of the feed element(en)
	 * @return string feed要素(ja) feed element(en)
	 * @access public
	 */
	function feed($content = null, $attributes = array()) {
		$attributes = $attributes + array('xmlns' => 'http://www.w3.org/2005/Atom');

		if(Configure::read('Config.language')) {
			$attributes['xml:lang'] = Configure::read('Config.language');
		}

		if(!isset($attributes['xml:base'])) {
			$attributes['xml:base'] = $this->currentUrl();
		}

		if(!$content) {
			$content = $this->getFeed();
		}

		return $this->tag('feed', $content, $attributes);
	}

	/**
	 * generator要素を生成します。(ja)
	 *
	 * Generate generator element.(en)
	 *
	 * <code>
	 * $atom->generator('Example Toolkit', array('uri' => 'http://www.example.com/', 'version' => '1.0'));
	 *
	 * # 出力すると以下のようになります。
	 * # <generator uri="http://www.example.com/" version="1.0">Example Toolkit</generator>
	 * </code>
	 *
	 * @param string $content generator要素の内容(ja) generator element content(en)
	 * @param array $attributes generator要素の属性(ja) The attributes of the generator element(en)
	 * @return string generator要素(ja) generator element(en)
	 * @access public
	 */
	function generator($content, $attributes = array()) {
		$attributes += array('#escape' => true, '#urn' => false);

		if(isset($attributes['uri']) && !$attributes['#urn']) {
			$attributes['uri'] = $this->url($attributes['uri'], true);
		}

		unset($attributes['#urn']);

		return $this->tag('generator', $content, $attributes);
	}

	/**
	 * このヘルパー内に溜められているentry要素以下を吐き出します。
	 *
	 * <code>
	 * $atom->getEntry();
	 * </code>
	 *
	 * @param boolean $clean 溜められているものを削除するか
	 * @return string entry要素内のXML要素
	 * @access public
  	 * @see setEntry()
	 */
	function getEntry($clean = true) {
		return $this->__getContent('_entry', $clean);
	}

	/**
	 * このヘルパー内に溜められているfeed要素以下を吐き出します。
	 *
	 * <code>
	 * $atom->getFeed();
	 * </code>
	 *
	 * @param boolean $clean 溜められているものを削除するか
	 * @return string feed要素内のXML要素
	 * @access public
  	 * @see setFeed()
	 */
	function getFeed($clean = true) {
		return $this->__getContent('_feed', $clean);
	}

	/**
	 * icon要素を生成します。(ja)
	 *
	 * Generate icon element.(en)
	 *
	 * <code>
	 * $atom->icon('/favicon.ico');
	 *
	 * # 出力すると以下のようになります。
	 * # <icon>http://example.com/favicon.ico</icon>
	 * </code>
	 *
	 * @param mixed $url アイコンへのURL(ja) Icon URL(en)
	 * @param array $attributes icon要素の属性(ja) The attributes of the icon element(en)
	 * @return string icon要素(ja) icon element(en)
	 * @access public
	 */
	function icon($url, $attributes = array()) {
		$attributes += array('#escape' => true, '#urn' => false);

		if(!$attributes['#urn']) {
			$url = $this->url($url, true);
		}

		unset($attributes['#urn']);

		return $this->tag('icon', $url, $attributes);
	}

	/**
	 * id要素を生成します。(ja)
	 *
	 * Generate id element.(en)
	 *
	 * <code>
	 * $atom->id('tag:example.org,2003:3', array('#urn' => true));
	 *
	 * # 出力すると以下のようになります。
	 * # <id>tag:example.org,2003:3</id>
	 * </code>
	 *
	 * @param mixed $id 識別子(ja) Identifier(en)
	 * @param array $attributes id要素の属性(ja) The attributes of the id element(en)
	 * @return string id要素(ja) id element(en)
	 * @access public
	 */
	function id($id = array(), $attributes = array()) {
		$attributes += array('#escape' => true, '#urn' => false);

		if(!$attributes['#urn']) {
			$id = $this->url($id, true);
		}

		unset($attributes['#urn']);

		return $this->tag('id', $id, $attributes);
	}

	/**
	 * link要素を生成します。(ja)
	 *
	 * Generate link element.(en)
	 *
	 * <code>
	 * $atom->link(array('controller' => 'posts', 'action' => 'index'));
	 *
	 * $atom->link(null, array('rel' => 'self'));
	 *
	 * $atom->link(
	 *     'http://example.org/audio/ph34r_my_podcast.mp3',
	 *     array('rel' => 'enclosure', 'type' => 'audio/mpeg', 'length' => 1337)
	 * );
	 *
	 * # 出力すると以下のようになります。
	 * # <link rel="alternate" href="http://example.com/posts" />
	 * # 
	 * # <link rel="self" href="http://example.com/" />
	 * # 
	 * # <link rel="enclosure" type="audio/mpeg" length="1337" href="http://example.org/audio/ph34r_my_podcast.mp3" />
	 * </code>
	 *
	 * @param mixed $url URL リンク先へのURL
	 * @param array $attributes link要素の属性(ja) The attributes of the link element(en)
	 * @return string link要素(ja) link element(en)
	 * @access public
	 */
	function link($url = null, $attributes = array()) {
		$attributes += array('#urn' => false);

		if(!isset($attributes['rel'])) {
			$attributes['rel'] = 'alternate';
		}

		if(!$url && $attributes['rel'] === 'self') {
			$url = $this->currentUrl();
		}

		if(!$url && isset($attributes['href'])) {
			$url = $attributes['href'];
		}

		if(!$attributes['#urn']) {
			$url = $this->url($url, true);
		}

		unset($attributes['#urn']);

		$attributes['href'] = $url;

		return $this->tag('link', null, $attributes);
	}

	/**
	 * logo要素を生成します。(ja)
	 *
	 * Generate logo element.(en)
	 *
	 * <code>
	 * $atom->logo('/img/logo.png');
	 *
	 * # 出力すると以下のようになります。
	 * # <logo>http://example.com/img/logo.png</logo>
	 * </code>
	 *
	 * @param mixed $url ロゴのURL(ja) Logo URL(en)
	 * @param array $attributes logo要素の属性(ja) The attributes of the logo element(en)
	 * @return string logo要素(ja) logo element(en)
	 * @access public
	 */
	function logo($url, $attributes = array()) {
		$attributes += array('#escape' => true, '#urn' => false);

		if(!$attributes['#urn']) {
			$url = $this->url($url, true);
		}

		unset($attributes['#urn']);

		return $this->tag('logo', $url, $attributes);
	}

	/**
	 * name要素を生成します。(ja)
	 *
	 * Generate name element.(en)
	 *
	 * <code>
	 * $atom->name('Mark Pilgrim');
	 *
	 * # 出力すると以下のようになります。
	 * # <name>Mark Pilgrim</name>
	 * </code>
	 *
	 * @param string $content name要素の内容(ja) name element content(en)
	 * @param array $attributes name要素の属性(ja) The attributes of the name element(en)
	 * @return string name要素(ja) name element(en)
	 * @access public
	 */
	function name($content, $attributes = array()) {
		$attributes += array('#escape' => true);
		return $this->tag('name', $content, $attributes);
	}

	/**
	 * atomPersonConstructとして処理し任意の要素を生成します。(ja)
	 *
	 * <code>
	 * $atom->personConstruct('author',array(
	 *     'name' => 'Sam Ruby'
	 * ));
	 *
	 * # 出力すると以下のようになります。
	 * # <author>
	 * #     <name>Sam Ruby</name>
	 * # </author>
	 * </code>
	 *
	 * @param mixed $name 要素の名前(ja) The name of the element(en)
	 * @param mixed $content 要素の内容 配列または文字列(ja) element content. array or string(en)
	 * @param array $attributes 要素の属性(ja) The attributes of the element(en)
	 * @return string XMLの要素(ja) XML elements(en)
	 * @access public
	 */
	function personConstruct($name, $content = array(), $attributes = array()) {
		$result = '';

		if(is_array($content)) {
			foreach($content as $key => $value) {
				$result .= $this->{$key}($value);
			}
		} else {
			$result = $content;
		}

		$out = $this->tag($name, $result, $attributes);

		return $out;
	}

	/**
	 * published要素を生成します。(ja)
	 *
	 * Generate published element.(en)
	 *
	 * <code>
	 * $atom->published(1071318569);
	 *
	 * $atom->published('2003-12-13T08:29:29-04:00');
	 *
	 * # 出力すると以下のようになります。
	 * # <published>2003-12-13T08:29:29-04:00</published>
	 * </code>
	 *
	 * @param mixed $timestamp published要素の内容。タイムスタンプまたはAtomの日付形式(ja) published element content. timestamp or Atom date format(en)
	 * @param array $attributes published要素の属性(ja) The attributes of the published element(en)
	 * @return string published要素(ja) published element(en)
	 * @access public
	 */
	function published($timestamp = null, $attributes = array()) {
		return $this->tag('published', $this->date($timestamp), $attributes);
	}

	/**
	 * rights要素を生成します。(ja)
	 *
	 * Generate rights element.(en)
	 *
	 * <code>
	 * $atom->rights('Copyright (c) 2003, Mark Pilgrim');
	 *
	 * # 出力すると以下のようになります。
	 * # <rights type="text">Copyright (c) 2003, Mark Pilgrim</rights>
	 * </code>
	 *
	 * @param string $content rights要素の内容(ja) rights element content(en)
	 * @param array $attributes rights要素の属性(ja) The attributes of the rights element(en)
	 * @return string rights要素(ja) rights element(en)
	 * @access public
	 */
	function rights($content, $attributes = array()) {
		return $this->textConstruct('rights', $content, $attributes);
	}

	/**
	 * 多次元配列をXMLに変形します。(ja)
	 *
	 * The multidimensional array is transformed into XML.(en)
	 *
	 * <code>
	 * $atom->serialize(array(
	 *     array('header'),
	 *     array('stylesheet', '/xsl/atom.xsl', array('charset' => 'UTF-8', 'type' => 'text/xsl')),
	 *     array('feed', array(
	 *         array('title', 'dive into mark'),
	 *         array('subtitle', 'A <em>lot</em> of effort went into making this effortless', array('type' => 'html')),
	 *         array('updated', '2003-12-13T18:30:02Z'),
	 *         array('id', 'tag:example.org,2003:3', array('#urn' => true)),
	 *         array('link', '/', array('type' => 'text/html')),
	 *         array('link', null, array('rel' => 'self', 'type' => 'application/atom+xml')),
	 *         array('rights', 'Copyright (c) 2003, Mark Pilgrim'),
	 *         array('generator', 'Example Toolkit', array('uri' => 'http://www.example.com/', 'version' => '1.0')),
	 *         array('dc:description', 'A lot of effort went into making this effortless'),
	 *         '<dc:format>application/atom+xml</dc:format>',
	 *         '<dc:date>2003-12-13T18:30:02Z</dc:date>',
	 *         array('entry', array(
	 *             array('title', 'Atom draft-07 snapshot'),
	 *             array('link', 'http://example.org/2003/12/13/atom03', array('type' => 'text/html')),
	 *             array('link', 'http://example.org/audio/ph34r_my_podcast.mp3', array('rel' => 'enclosure', 'type' => 'audio/mpeg', 'length' => '1337')),
	 *             array('id', 'tag:example.org,2003:3.2397', array('#urn' => true)),
	 *             array('updated', '2005-07-31T12:29:29Z'),
	 *             array('published', 1071318569),
	 *             array('author', array('name' => 'John Doe', 'uri' => 'http://example.org/', 'email' => 'f8dy@example.com')),
	 *             array('contributor', array('name' => 'Sam Ruby')),
	 *             array('contributor', array('name' => 'Joe Gregorio')),
	 *             array('content', '<p><i>[Update: The Atom draft is finished.]</i></p>', array('type' => 'xhtml', 'xml:lang' => 'en', 'xml:base' => 'http://diveintomark.org/'))
	 *         ))
	 *     ),
	 *         array('xmlns:dc' => 'http://purl.org/dc/elements/1.1/')
	 *     )
	 * ));
	 *
	 * # 出力すると以下のようになります。
	 * # <?xml version="1.0" encoding="UTF-8" ?>
	 * # <?xml-stylesheet charset="UTF-8" type="text/xsl" href="http://example.com/xsl/atom.xsl" ?>
	 * # <feed xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns="http://www.w3.org/2005/Atom" xml:lang="en" xml:base="http://example.com/atom">
	 * #     <title type="text">dive into mark</title>
	 * #     <subtitle type="html">A &lt;em&gt;lot&lt;/em&gt; of effort went into making this effortless</subtitle>
	 * #     <updated>2003-12-13T18:30:02Z</updated>
	 * #     <id>tag:example.org,2003:3</id>
	 * #     <link type="text/html" rel="alternate" href="http://example.com/" />
	 * #     <link rel="self" type="application/atom+xml" href="http://example.com/atom" />
	 * #     <rights type="text">Copyright (c) 2003, Mark Pilgrim</rights>
	 * #     <generator uri="http://www.example.com/" version="1.0">Example Toolkit</generator>
	 * #     <dc:description>A lot of effort went into making this effortless</dc:description>
	 * #     <dc:format>application/atom+xml</dc:format>
	 * #     <dc:date>2003-12-13T18:30:02Z</dc:date>
	 * #     <entry>
	 * #         <title type="text">Atom draft-07 snapshot</title>
	 * #         <link type="text/html" rel="alternate" href="http://example.org/2003/12/13/atom03" />
	 * #         <link rel="enclosure" type="audio/mpeg" length="1337" href="http://example.org/audio/ph34r_my_podcast.mp3" />
	 * #         <id>tag:example.org,2003:3.2397</id>
	 * #         <updated>2005-07-31T12:29:29Z</updated>
	 * #         <published>2003-12-13T12:29:29+00:00</published>
	 * #         <author>
	 * #             <name>John Doe</name>
	 * #             <uri>http://example.org/</uri>
	 * #             <email>f8dy@example.com</email>
	 * #         </author>
	 * #         <contributor>
	 * #             <name>Sam Ruby</name>
	 * #         </contributor>
	 * #         <contributor>
	 * #             <name>Joe Gregorio</name>
	 * #         </contributor>
	 * #         <content type="xhtml" xml:lang="en" xml:base="http://diveintomark.org/">
	 * #             <div xmlns="http://www.w3.org/1999/xhtml">
	 * #                 <p><i>[Update: The Atom draft is finished.]</i></p>
	 * #             </div>
	 * #         </content>
	 * #     </entry>
	 * # </feed>
	 * </code>
	 *
	 * @param array $data 多次元配列(ja) Multidimensional array(en)
	 * @return string XMLの要素(ja) XML elements(en)
	 * @access public
	 */
	function serialize($data) {
		$out = '';

		foreach($data as $key => $content) {
			if(is_array($content)) {
				$name = array_shift($content);

				unset($content['name']);

				if(method_exists($this, $name)) {
					$out .= call_user_func_array(array($this, $name), $content);
				} else {
					array_unshift($content, $name);

					$out .= call_user_func_array(array($this, 'tag'), $content);
				}
			} else {
				$out .= $content;
			}
		}

		return $out;
	}

	/**
	 * ヘルパー内にentry要素以下の要素を溜めます。
	 *
	 * <code>
	 * $atom->setEntry('title', 'Example Page');
	 *
	 * $atom->setEntry('link', array('controller' => 'posts', 'action' => 'index'));
	 *
	 * //以下のような書き方も可能です。
	 * $atom->setEntryTitle('Example Page');
	 *
	 * $atom->setEntryLink(array('controller' => 'posts', 'action' => 'index'));
	 * </code>
	 *
	 * @return void
	 * @access public
	 * @see getEntry()
	 */
	function setEntry() {
		$arguments = func_get_args();
		$name = array_shift($arguments);
		$this->_entry[] = compact('name', 'arguments');
	}

	/**
	 * ヘルパー内にfeed要素以下の要素を溜めます。
	 *
	 * <code>
	 * $atom->setFeed('title', 'Example Feed');
	 *
	 * $atom->setFeed('link', array('controller' => 'posts', 'action' => 'index'));
	 *
	 * //以下のような書き方も可能です。
	 * $atom->setFeedTitle('Example Feed');
	 *
	 * $atom->setFeedLink(array('controller' => 'posts', 'action' => 'index'));
	 * </code>
	 *
	 * @return void
	 * @access public
	 * @see getFeed()
	 */
	function setFeed() {
		$arguments = func_get_args();
		$name = array_shift($arguments);
		$this->_feed[] = compact('name', 'arguments');
	}

	/**
	 * 開始タグを生成します。(ja)
	 *
	 * Generate start tag.(en)
	 *
	 * <code>
	 * $atom->startTag('feed');
	 *
	 * $atom->startTag('entry');
	 *
	 * # 出力すると以下のようになります。
	 * # <feed>
	 * #
	 * # <entry>
	 * </code>
	 *
	 * @param string $name 要素の名前(ja) The name of the start tag(en)
	 * @param array $attributes 開始タグの属性(ja) The attributes of the start tag(en)
	 * @return string 開始タグ(ja) start tag(en)
	 * @access public
	 */
	function startTag($name, $attributes = array()) {
		$out = '<';

		$out .= $name;

		if($attributes) {
			$out .= ' ' . $this->_attributes($attributes);
		}

		$out .= '>';

		return $out;
	}

	/**
	 * xml-stylesheetを生成します。(ja)
	 *
	 * Generate xml-stylesheet.(en)
	 *
	 * <code>
	 * $atom->stylesheet('/xsl/atom.xsl', array('charset' => 'UTF-8', 'type' => 'text/xsl'));
	 *
	 * # 出力すると以下のようになります。
	 * # <?xml-stylesheet charset="UTF-8" type="text/xsl" href="http://example.com/xsl/atom.xsl" ?>
	 * </code>
	 *
	 * @param string $url スタイルシートへのURL(ja) Stylesheet URL(en)
	 * @param array $attributes xml-stylesheetの属性(ja) The attributes of the xml-stylesheet(en)
	 * @return string xml-stylesheet
	 * @access public
	 */
	function stylesheet($url = null, $attributes = array()) {
		if(!$url && isset($attributes['href'])) {
			$url = $attributes['href'];
		}

		$attributes['href'] = $this->url($url, true);

		return $this->header('xml-stylesheet' . $this->_attributes($attributes));
	}

	/**
	 * subtitle要素を生成します。(ja)
	 *
	 * Generate subtitle element.(en)
	 *
	 * <code>
	 * $atom->subtitle('A <em>lot</em> of effort', array('type' => 'html'));]
	 *
	 * # 出力すると以下のようになります。
	 * # <subtitle type="html">A &lt;em&gt;lot&lt;/em&gt; of effort</subtitle>
	 * </code>
	 *
	 * @param string $content subtitle要素の内容(ja) subtitle element content(en)
	 * @param array $attributes subtitle要素の属性(ja) The attributes of the subtitle element(en)
	 * @return string subtitle要素(ja) subtitle element(en)
	 * @access public
	 */
	function subtitle($content, $attributes = array()) {
		return $this->textConstruct('subtitle', $content, $attributes);
	}

	/**
	 * summary要素を生成します。(ja)
	 *
	 * Generate summary element.(en)
	 *
	 * <code>
	 * $atom->summary('Some text.');
	 *
	 * # 出力すると以下のようになります。
	 * # <summary type="text">Some text.</summary>
	 * </code>
	 *
	 * @param string $content summary要素の内容(ja) summary element content(en)
	 * @param array $attributes summary要素の属性(ja) The attributes of the summary element(en)
	 * @return string summary要素(ja) summary element(en)
	 * @access public
	 */
	function summary($content, $attributes = array()) {
		return $this->textConstruct('summary', $content, $attributes);
	}

	/**
	 * XMLの要素を生成します。(ja)
	 *
	 * Generate XML element.(en)
	 *
	 * <code>
	 * $atom->tag('dc:description', 'Example Page', array('xmlns:dc' => 'http://purl.org/dc/elements/1.1/'));
	 *
	 * //以下のようにも記述できます。
	 * $atom->description('Example Page', array('xmlns:dc' => 'http://purl.org/dc/elements/1.1/'), 'dc');
	 *
	 * # 出力すると以下のようになります。
	 * # <dc:description xmlns:dc="http://purl.org/dc/elements/1.1/">Example Page</dc:description>
	 * </code>
	 *
	 * @param string $name 要素の名前(ja) The name of the XML element(en)
	 * @param string $content 要素の内容(ja) The attributes of the XML element(en)
	 * @param array $attributes 要素の属性(ja) XML element content(en)
	 * @param string $namespace 要素の名前空間(ja) The namespace of the XML element(en)
	 * @return string XMLの要素(ja) XML element(en)
	 * @access public
	 */
	function tag($name = null, $content = null, $attributes = array(), $namespace = false) {
		$attributes = $attributes + array('#escape' => false, '#cdata' => false);

		if(is_array($content)) {
			$content = $this->serialize($content);
		}

		if($content && $attributes['#escape']) {
			$content = h($content);
		}
		unset($attributes['#escape']);

		$cdata = false;
		if(isset($attributes['#cdata'])) {
			$cdata = $attributes['#cdata'];
			unset($attributes['#cdata']);
		}

		if($name) {
			$node = $this->Xml->createElement($name, $content, array(), $namespace);

			foreach($attributes as $attKey => $attValue) {
				$node->attributes[$attKey] = $attValue;
			}
		} else {
			$node = $this->Xml->createTextNode($content);
		}

		return $node->toString(array('cdata' => $cdata));
	}

	/**
	 * atomTextConstructとして処理し任意の要素を生成します
	 *
	 * <code>
	 * $atom->textConstruct(
	 *     'subtitle',
	 *     'A <em>lot</em> of effort went into making this effortless',
	 *     array('type' => 'html')
	 * );
	 *
	 * # 出力すると以下のようになります。
	 * # <subtitle type="html">A &lt;em&gt;lot&lt;/em&gt; of effort went into making this effortless</subtitle>
	 * </code>
	 *
	 * @param string $name 要素の名前(ja) The name of the element(en)
	 * @param string $content 要素の内容(ja) element content(en)
	 * @param array $attributes 要素の属性(ja) The attributes of the element(en)
	 * @return string XMLの要素(ja) XML elements(en)
	 * @access public
	 */
	function textConstruct($name, $content, $attributes = array()) {
		$attributes += array('#escape' => true);

		if(!isset($attributes['type'])) {
			$attributes['type'] = 'text';
		} elseif($attributes['type'] === 'xhtml') {
			$content = $this->tag('div', $content, array('xmlns' => 'http://www.w3.org/1999/xhtml'));
			$attributes['#escape'] = false;
		}

		return $this->tag($name, $content, $attributes);
	}

	/**
	 * title要素を生成します。(ja)
	 *
	 * Generate title element.(en)
	 *
	 * <code>
	 * $atom->title('Example Feed');
	 *
	 * # 出力すると以下のようになります。
	 * # <title type="text">Example Feed</title>
	 * </code>
	 *
	 * @param string $content title要素の内容(ja) title element content(en)
	 * @param array $attributes title要素の属性(ja) The attributes of the title element(en)
	 * @return string title要素(ja) title element(en)
	 * @access public
	 */
	function title($content, $attributes = array()) {
		return $this->textConstruct('title', $content, $attributes);
	}

	/**
	 * updated要素を生成します。(ja)
	 *
	 * Generate updated element.(en)
	 *
	 * <code>
	 * $atom->updated(1071340202);
	 *
	 * $atom->updated('2003-12-13T18:30:02Z');
	 *
	 * # 出力すると以下のようになります。
	 * # <updated>2003-12-13T18:30:02Z</updated>
	 * </code>
	 *
	 * @param mixed $timestamp updated要素の内容。タイムスタンプまたはAtomの日付形式(ja) updated element content. timestamp or Atom date format(en)
	 * @param array $attributes updated要素の属性(ja) The attributes of the updated element(en)
	 * @return string updated要素(ja) updated element(en)
	 * @access public
	 */
	function updated($timestamp = null, $attributes = array()) {
		return $this->tag('updated', $this->date($timestamp), $attributes);
	}

	/**
	 * uri要素を生成します。(ja)
	 *
	 * Generate uri element.(en)
	 *
	 * <code>
	 * $atom->uri('http://example.org/');
	 *
	 * # 出力すると以下のようになります。
	 * # <uri>http://example.org/</uri>
	 * </code>
	 *
	 * @param mixed $url uri要素の内容(ja) uri element content(en)
	 * @param array $attributes uri要素の属性(ja) The attributes of the uri element(en)
	 * @return string uri要素(ja) uri element(en)
	 * @access public
	 */
	function uri($url, $attributes = array()) {
		$attributes += array('#escape' => true, '#urn' => false);

		if(!$attributes['#urn']) {
			$url = $this->url($url, true);
		}

		unset($attributes['#urn']);

		return $this->tag('uri', $url, $attributes);
	}

	/**
	 * Finds URL for specified action.
	 *
	 * Returns a URL pointing at the provided parameters.
	 *
	 * @param mixed $url Either a relative string url like `/products/view/23` or
	 * an array of url parameters. Using an array for urls will allow you to leverage
	 * the reverse routing features of CakePHP.
	 * @param boolean $full If true, the full base URL will be prepended to the result
	 * @return string Full translated URL with base path.
	 * @access public
	 * @link http://book.cakephp.org/view/1448/url
	 */
	function url($url = null, $full = false) {
		return html_entity_decode(parent::url($url, $full), ENT_QUOTES, Configure::read('App.encoding'));
	}

	/**
	 * 配列の属性を文字列化します。
	 *
	 * @param array $attribute 属性(ja) attributes(en)
	 * @return string 属性(ja) attributes(en)
	 * @access protected
	 */
	function _attributes($attributes = array()) {
		$out = '';

		$attributes = array_filter($attributes);

		foreach($attributes as $key => $content) {
			$out .= ' ' . $key . '="' .  h($content) . '"';
		}

		return $out;
	}

	function __call($name, $arguments) {
		if(preg_match('/^(setFeed|setEntry)([A-Z].+?)$/', $name, $matches)) {
			array_unshift($arguments, lcfirst($matches[2]));

			return call_user_func_array(array($this, $matches[1]), $arguments);
		}
	
		array_unshift($arguments, $name);

		return call_user_func_array(array($this, 'tag'), $arguments);
	}

	/**
	 * ヘルパー内に溜めてあるものを取り出します。
	 *
	 * @param string $name 取り出すメンバー変数
 	 * @param boolean $clean 溜めてあるものを消去するか
	 * @return string XMLの要素
	 * @access private
	 */
	function __getContent($name, $clean = false) {
		$out = '';

		foreach($this->{$name} as $key => $content) {
			if($content['name'] === '#text') {
				$out .= implode($content['arguments']);
			} else {
				$out .= call_user_func_array(array($this, $content['name']), $content['arguments']);
			}
		}

		if($clean) {
			$this->{$name} = array();
		}

		return $out;
	}
}
?>